@@ -14,13 +14,19 @@
 # under the License.
 
 from pecan import expose, response
-from v1 import V1Controller
+from v1.v1 import V1Controller
+from v2.v2_0 import V2Controller
 from libra.admin_api.model.responses import Responses
 
 
 class RootController(object):
     """root control object."""
 
+    @expose('json')
+    def index(self):
+        response.status = 200
+        return Responses.versions
+
     @expose('json')
     def _default(self):
         """default route.. acts as catch all for any wrong urls.
@@ -32,6 +38,8 @@ class RootController(object):
     def _lookup(self, primary_key, *remainder):
         if primary_key == 'v1':
             return V1Controller(), remainder
+        if primary_key == 'v2.0':
+            return V2Controller(), remainder
         else:
             response.status = 404
             return Responses._default
