@@ -27,13 +27,13 @@ from logs import LogsController
 
 # models
 from libra.common.api.lbaas import LoadBalancer, Device, Node, db_session
-from libra.common.api.lbaas import loadbalancers_devices, Limits, Vip
-from libra.common.api.lbaas import HealthMonitor
+from libra.common.api.lbaas import TenantLimits
+from libra.common.api.lbaas import loadbalancers_devices, Limits, Vip, Ports
+from libra.common.api.lbaas import HealthMonitor, Counters
 from libra.common.exc import ExhaustedError
 from libra.api.model.validators import LBPut, LBPost, LBResp, LBVipResp
-from libra.api.model.validators import LBRespNode
+from libra.api.model.validators import LBRespNode, LBOptions
 from libra.common.api.gearman_client import submit_job
-from libra.common.api.mnb import update_mnb
 from libra.api.acl import get_limited_to_project
 from libra.api.library.exp import OverLimit, IPOutOfRange, NotFound
 from libra.api.library.exp import ImmutableEntity, ImmutableStates
@@ -44,6 +44,12 @@ from wsme import types as wtypes
 
 
 class LoadBalancersController(RestController):
+
+    LB_TIMEOUT_MS = 30000
+    LB_TIMEOUT_MAX = 1000000
+    LB_RETRIES = 3
+    LB_RETRIES_MAX = 256
+
     def __init__(self, lbid=None):
         self.lbid = lbid
 
@@ -77,7 +83,8 @@ class LoadBalancersController(RestController):
                         LoadBalancer.protocol,
                         LoadBalancer.port, LoadBalancer.algorithm,
                         LoadBalancer.status, LoadBalancer.created,
-                        LoadBalancer.updated
+                        LoadBalancer.updated, LoadBalancer.timeout,
+                        LoadBalancer.retries
                     ).filter(LoadBalancer.tenantid == tenant_id).\
                         filter(LoadBalancer.status == 'DELETED').all()
                 else:
@@ -86,7 +93,8 @@ class LoadBalancersController(RestController):
                         LoadBalancer.protocol,
                         LoadBalancer.port, LoadBalancer.algorithm,
                         LoadBalancer.status, LoadBalancer.created,
-                        LoadBalancer.updated
+                        LoadBalancer.updated, LoadBalancer.timeout,
+                        LoadBalancer.retries
                     ).filter(LoadBalancer.tenantid == tenant_id).\
                         filter(LoadBalancer.status != 'DELETED').all()
                 load_balancers = {'loadBalancers': []}
@@ -96,6 +104,20 @@ class LoadBalancersController(RestController):
                     lb['nodeCount'] = session.query(Node).\
                         filter(Node.lbid == lb['id']).count()
                     lb['id'] = str(lb['id'])
+
+                    # Unset options get set to default values
+                    lb['options'] = {}
+                    if lb['timeout']:
+                        lb['options']['timeout'] = lb['timeout']
+                    else:
+                        lb['options']['timeout'] = self.LB_TIMEOUT_MS
+                    if lb['retries']:
+                        lb['options']['retries'] = lb['retries']
+                    else:
+                        lb['options']['retries'] = self.LB_RETRIES
+                    del(lb['timeout'])
+                    del(lb['retries'])
+
                     load_balancers['loadBalancers'].append(lb)
             else:
                 load_balancers = session.query(
@@ -103,6 +125,7 @@ class LoadBalancersController(RestController):
                     LoadBalancer.port, LoadBalancer.algorithm,
                     LoadBalancer.status, LoadBalancer.created,
                     LoadBalancer.updated, LoadBalancer.errmsg,
+                    LoadBalancer.timeout, LoadBalancer.retries,
                     Vip.id.label('vipid'), Vip.ip
                 ).join(LoadBalancer.devices).\
                     outerjoin(Device.vip).\
@@ -168,7 +191,28 @@ class LoadBalancersController(RestController):
                         del node['weight']
                     load_balancers['nodes'].append(node)
 
-            session.rollback()
+                # Unset options get set to default values
+                load_balancers['options'] = {}
+                if load_balancers['timeout']:
+                    load_balancers['options']['timeout'] =\
+                        load_balancers['timeout']
+                else:
+                    load_balancers['options']['timeout'] = self.LB_TIMEOUT_MS
+                if load_balancers['retries']:
+                    load_balancers['options']['retries'] =\
+                        load_balancers['retries']
+                else:
+                    load_balancers['options']['retries'] = self.LB_RETRIES
+                del(load_balancers['timeout'])
+                del(load_balancers['retries'])
+
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_loadbalancers_get').first()
+                counter.value += 1
+                session.commit()
+            except:
+                pass
             response.status = 200
             return load_balancers
 
@@ -251,6 +295,35 @@ class LoadBalancersController(RestController):
             if is_galera and not is_backup:
                 num_galera_primary_nodes += 1
 
+        # Options defaults
+        timeout_ms = self.LB_TIMEOUT_MS
+        retries = self.LB_RETRIES
+        if body.options:
+            if body.options.timeout != Unset:
+                try:
+                    timeout_ms = int(body.options.timeout)
+                    if timeout_ms < 0 or timeout_ms > self.LB_TIMEOUT_MAX:
+                        raise ClientSideError(
+                            'timeout must be between 0 and {0} ms'
+                            .format(self.LB_TIMEOUT_MAX)
+                        )
+                except ValueError:
+                    raise ClientSideError(
+                        'timeout must be an integer'
+                    )
+            if body.options.retries != Unset:
+                try:
+                    retries = int(body.options.retries)
+                    if retries < 0 or retries > self.LB_RETRIES_MAX:
+                        raise ClientSideError(
+                            'retries must be between 0 and {0}'
+                            .format(self.LB_RETRIES_MAX)
+                        )
+                except ValueError:
+                    raise ClientSideError(
+                        'retries must be an integer'
+                    )
+
         # Galera sanity checks
         if is_galera and num_galera_primary_nodes != 1:
             raise ClientSideError(
@@ -267,6 +340,15 @@ class LoadBalancersController(RestController):
             count = session.query(LoadBalancer).\
                 filter(LoadBalancer.tenantid == tenant_id).\
                 filter(LoadBalancer.status != 'DELETED').count()
+            ports = session.query(Ports.protocol, Ports.portnum).\
+                filter(Ports.enabled == 1).all()
+
+            # Allow per-tenant LB limit, defaulting to the global limit if
+            # the per-tenant value is not set.
+            tenant_lblimit = session.query(TenantLimits.loadbalancers).\
+                filter(TenantLimits.tenantid == tenant_id).scalar()
+            if tenant_lblimit:
+                lblimit = tenant_lblimit
 
             if len(body.name) > namelimit:
                 session.rollback()
@@ -308,7 +390,20 @@ class LoadBalancersController(RestController):
                     raise ClientSideError(
                         'Port number {0} is invalid'.format(body.port)
                     )
-                lb.port = body.port
+                # Make sure the port is valid and enabled
+                valid = False
+                for item in ports:
+                    item = item._asdict()
+                    if(lb.protocol == item["protocol"].upper() and
+                            body.port == item["portnum"]):
+                        valid = True
+                if valid:
+                    lb.port = body.port
+                else:
+                    raise ClientSideError(
+                        'Port number {0} is not allowed for {1} protocol'
+                        .format(body.port, lb.protocol)
+                    )
             else:
                 if lb.protocol == 'HTTP':
                     lb.port = 80
@@ -385,21 +480,53 @@ class LoadBalancersController(RestController):
                     count()
                 if old_count:
                     session.rollback()
-                    # Error here, can have only one LB per port on a device
+                    # Error, can have only one LB per port on a device
                     raise ClientSideError(
                         'Only one load balancer per port allowed per device'
                     )
 
+                if lb.protocol == 'HTTP':
+                    protocol_count = session.query(
+                        LoadBalancer
+                    ).join(LoadBalancer.devices).\
+                        join(Device.vip).\
+                        filter(LoadBalancer.tenantid == tenant_id).\
+                        filter(Vip.id == virtual_id).\
+                        filter(LoadBalancer.protocol == lb.protocol).\
+                        count()
+                else:
+                    # TCP or GALERA. Both are TCP really
+                    protocol_count = session.query(
+                        LoadBalancer
+                    ).join(LoadBalancer.devices).\
+                        join(Device.vip).\
+                        filter(LoadBalancer.tenantid == tenant_id).\
+                        filter(Vip.id == virtual_id).\
+                        filter((LoadBalancer.protocol == 'TCP') |
+                               (LoadBalancer.protocol == 'GALERA')).\
+                        count()
+
+                if protocol_count:
+                    session.rollback()
+                    # Error, can have only one LB per protocol on a device
+                    raise ClientSideError(
+                        'Only one load balancer per protocol'
+                        ' allowed per device'
+                    )
+
             if body.algorithm:
                 lb.algorithm = body.algorithm.upper()
             else:
                 lb.algorithm = 'ROUND_ROBIN'
 
+            lb.timeout = timeout_ms
+            lb.retries = retries
+
             lb.devices = [device]
             # write to database
             session.add(lb)
             session.flush()
-            #refresh the lb record so we get the id back
+            # refresh the lb record so we get the id back
             session.refresh(lb)
             for node in body.nodes:
                 if node.condition == 'DISABLED':
@@ -450,29 +577,54 @@ class LoadBalancersController(RestController):
                     address=None, id=None, type='ASSIGNING', ipVersion='IPV4'
                 )
             return_data.virtualIps = [vip_resp]
+
+            nodes = session.query(
+                Node.id, Node.address, Node.port, Node.status,
+                Node.enabled, Node.weight
+            ).join(LoadBalancer.nodes).\
+                filter(LoadBalancer.tenantid == tenant_id).\
+                filter(LoadBalancer.id == lb.id).\
+                all()
+
             return_data.nodes = []
-            for node in body.nodes:
-                if node.weight != Unset and node.weight != 1:
-                    out_node = LBRespNode(
-                        port=str(node.port), address=node.address,
-                        condition=node.condition, weight=weight
+            for node in nodes:
+                if node.enabled == 1:
+                    condition = 'ENABLED'
+                else:
+                    condition = 'DISABLED'
+
+                if node.weight == 1:
+                    return_data.nodes.append(
+                        LBRespNode(
+                            id=str(node.id), port=str(node.port),
+                            address=node.address, condition=condition,
+                            status=node.status
+                        )
                     )
                 else:
-                    out_node = LBRespNode(
-                        port=str(node.port), address=node.address,
-                        condition=node.condition
+                    return_data.nodes.append(
+                        LBRespNode(
+                            id=str(node.id), port=str(node.port),
+                            address=node.address, condition=condition,
+                            status=node.status, weight=str(node.weight)
+                        )
                     )
 
-                return_data.nodes.append(out_node)
-            session.commit()
+            return_data.options = LBOptions(timeout=timeout_ms,
+                                            retries=retries)
+
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_loadbalancers_create').first()
+                counter.value += 1
+                session.commit()
+            except:
+                pass
             # trigger gearman client to create new lb
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
             )
 
-            #Notify billing of the LB creation
-            update_mnb('lbaas.instance.create', lb.id, tenant_id)
-
             return return_data
 
     @wsme_pecan.wsexpose(None, body=LBPut, status_code=202)
@@ -513,18 +665,53 @@ class LoadBalancersController(RestController):
             if body.algorithm != Unset:
                 lb.algorithm = body.algorithm
 
+            if body.options:
+                if body.options.timeout != Unset:
+                    try:
+                        timeout_ms = int(body.options.timeout)
+                        if timeout_ms < 0 or timeout_ms > self.LB_TIMEOUT_MAX:
+                            raise ClientSideError(
+                                'timeout must be between 0 and {0} ms'
+                                .format(self.LB_TIMEOUT_MAX)
+                            )
+                        lb.timeout = timeout_ms
+                    except ValueError:
+                        raise ClientSideError(
+                            'timeout must be an integer'
+                        )
+                if body.options.retries != Unset:
+                    try:
+                        retries = int(body.options.retries)
+                        if retries < 0 or retries > self.LB_RETRIES_MAX:
+                            raise ClientSideError(
+                                'retries must be between 0 and {0}'
+                                .format(self.LB_RETRIES_MAX)
+                            )
+                        lb.retries = retries
+                    except ValueError:
+                        raise ClientSideError(
+                            'retries must be an integer'
+                        )
+
             lb.status = 'PENDING_UPDATE'
             device = session.query(
                 Device.id, Device.name, Device.status
             ).join(LoadBalancer.devices).\
                 filter(LoadBalancer.id == self.lbid).\
                 first()
-
             session.commit()
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
             )
-            return ''
+        with db_session() as session:
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_loadbalancers_modify').first()
+                counter.value += 1
+                session.commit()
+            except:
+                pass
+        return ''
 
     @wsme_pecan.wsexpose(None, status_code=202)
     def delete(self):
@@ -584,9 +771,16 @@ class LoadBalancersController(RestController):
                     'DELETE', device.name, device.id, lb.id
                 )
 
-            #Notify billing of the LB deletion
-            update_mnb('lbaas.instance.delete', lb.id, tenant_id)
-            return None
+        with db_session() as session:
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_loadbalancers_delete').first()
+                counter.value += 1
+                session.commit()
+            except:
+                pass
+
+        return None
 
     def usage(self, load_balancer_id):
         """List current and historical usage.
