@@ -27,10 +27,12 @@ class LBNode(Base):
 
 
 class LBRespNode(Base):
+    id = wtypes.text
     port = wtypes.text
     address = wtypes.text
     condition = wtypes.text
-    weight = int
+    status = wtypes.text
+    weight = wtypes.text
 
 
 class LBNodePut(Base):
@@ -59,9 +61,15 @@ class LBVip(Base):
     id = wsattr(int, mandatory=True)
 
 
+class LBOptions(Base):
+    timeout = int
+    retries = int
+
+
 class LBPost(Base):
     name = wsattr(wtypes.text, mandatory=True)
     nodes = wsattr(['LBNode'], mandatory=True)
+    options = wsattr('LBOptions')
     protocol = wtypes.text
     algorithm = Enum(wtypes.text, 'ROUND_ROBIN', 'LEAST_CONNECTIONS')
     port = int
@@ -71,6 +79,7 @@ class LBPost(Base):
 class LBPut(Base):
     name = wtypes.text
     algorithm = Enum(wtypes.text, 'ROUND_ROBIN', 'LEAST_CONNECTIONS')
+    options = wsattr('LBOptions')
 
 
 class LBVipResp(Base):
@@ -98,6 +107,7 @@ class LBResp(Base):
     updated = wtypes.text
     virtualIps = wsattr(['LBVipResp'])
     nodes = wsattr(['LBRespNode'])
+    options = wsattr('LBOptions')
 
 
 class LBMonitorPut(Base):
