@@ -70,10 +70,10 @@ class LBaaSController(object):
                 return self._action_discover()
             elif action == 'ARCHIVE':
                 return self._action_archive()
+            elif action == 'METRICS':
+                return self._action_metrics()
             elif action == 'STATS':
                 return self._action_stats()
-            elif action == 'PING':
-                return self._action_ping()
             elif action == 'DIAGNOSTICS':
                 return self._action_diagnostic()
             else:
@@ -133,6 +133,45 @@ class LBaaSController(object):
             )
             raise
 
+    def _set_lb_options(self, protocol, options):
+        """
+        Parse load balancer options.
+
+        options
+            Dictionary of load balancer options.
+
+        Returns: True on success, False otherwise
+        """
+
+        # Default timeout values in milliseconds
+        client_val = 30000
+        server_val = 30000
+        connect_val = 30000
+        retries_val = 3
+
+        if 'client_timeout' in options:
+            client_val = options['client_timeout']
+        if 'server_timeout' in options:
+            server_val = options['server_timeout']
+        if 'connect_timeout' in options:
+            connect_val = options['connect_timeout']
+        if 'connect_retries' in options:
+            retries_val = options['connect_retries']
+
+        try:
+            self.driver.set_timeouts(protocol, client_val, server_val,
+                                     connect_val, retries_val)
+        except NotImplementedError:
+            pass
+        except Exception as e:
+            error = "Failed to set timeout values: %s" % e
+            LOG.error(error)
+            self.msg[self.ERROR_FIELD] = error
+            self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+            return False
+
+        return True
+
     def _action_discover(self):
         """
         Return service discovery information.
@@ -257,6 +296,15 @@ class LBaaSController(object):
                     self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                     return self.msg
 
+            if 'options' in current_lb:
+                lb_options = current_lb['options']
+            else:
+                lb_options = {}
+
+            # Always call _set_lb_options() since it sets sensible defaults
+            if not self._set_lb_options(current_lb['protocol'], lb_options):
+                return self.msg
+
             for lb_node in current_lb['nodes']:
                 port = None
                 address = None
@@ -440,7 +488,7 @@ class LBaaSController(object):
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
         return self.msg
 
-    def _action_ping(self):
+    def _action_stats(self):
         """
         Get load balancer and node status.
 
@@ -452,18 +500,18 @@ class LBaaSController(object):
         try:
             nodes = self.driver.get_status()
         except NotImplementedError:
-            error = "Selected driver does not support PING action."
+            error = "Selected driver does not support STATS action."
             LOG.error(error)
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = error
         except DeletedStateError:
-            error = "Invalid operation PING on a deleted LB."
+            error = "Invalid operation STATS on a deleted LB."
             LOG.error(error)
             self.msg['status'] = 'DELETED'
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = error
         except Exception as e:
-            LOG.error("PING failed: %s, %s" % (e.__class__, e))
+            LOG.error("STATS failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = str(e)
         else:
@@ -474,9 +522,9 @@ class LBaaSController(object):
 
         return self.msg
 
-    def _action_stats(self):
+    def _action_metrics(self):
         """
-        Get load balancer statistics
+        Get load balancer metrics
 
         This type of request gets the number of bytes out for each load
         balancer defined on the device. If both a TCP and HTTP load
@@ -486,20 +534,20 @@ class LBaaSController(object):
         try:
             start, end, statistics = self.driver.get_statistics()
         except NotImplementedError:
-            error = "Selected driver does not support STATS action."
+            error = "Selected driver does not support METRICS action."
             LOG.error(error)
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = error
             return self.msg
         except DeletedStateError:
-            error = "Invalid operation STATS on a deleted LB."
+            error = "Invalid operation METRICS on a deleted LB."
             LOG.error(error)
             self.msg['status'] = 'DELETED'
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = error
             return self.msg
         except Exception as e:
-            LOG.error("STATS failed: %s, %s" % (e.__class__, e))
+            LOG.error("METRICS failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = str(e)
             return self.msg
